#include "RhdpdaUtils.h"

namespace determinization {
namespace rhdpda {


const Symbol RhdpdaUtils::BOTTOM_OF_STACK_SYMBOL("_"); // TODO specialni symbol pro dno zasobniku zpusobuje chybu


string RhdpdaUtils::buildPairName(const StateSymbolPair& pair)
{
    return "('" + pair.state.getName() + "', '" + pair.symbol.getSymbol() + "')";;
}


string RhdpdaUtils::buildStateName(const SComponent& s, const RComponent& r)
{
    string name = "{";
    for (const auto& pairOfPairs : s.pairsOfPairs) {
        if (&pairOfPairs != &(*begin(s.pairsOfPairs))) {
            name += ", ";
        }
        name += "(" + buildPairName(pairOfPairs.pair1) + ", " + buildPairName(pairOfPairs.pair2) + ")";
    }
    name += "}, {";
    for (const auto& pair : r.pairs) {
        if (&pair != &(*begin(r.pairs))) {
            name += ", ";
        }
        name += buildPairName(pair);
    }
    return name + "}";
}


const State& RhdpdaUtils::getOrCreateState(const SComponent& s, const RComponent& r, map<string, StateData>& states,
    PDA& rdpda)
{
    string stateName = buildStateName(s, r);

    map<string, StateData>::iterator statesIter = states.find(stateName);
    if (statesIter != states.end()) {
        return statesIter->second.state;
    }
    
    State state(stateName);
    StateData stateData(state, s, r);
    StateData& insertedData = states.insert(pair<string, StateData>(stateName, stateData)).first->second;
    rdpda.addState(state);
    return insertedData.state;
}


const SComponent RhdpdaUtils::getSComponentWithPairsIdentity(const set<StateSymbolPair>& pairs)
{
    set<PairOfPairs> pairsOfPairs;
    for (const auto& pair : pairs) {
        pairsOfPairs.insert(PairOfPairs(pair, pair));
    }
    return SComponent(pairsOfPairs);
}


string RhdpdaUtils::buildStackSymbolName(const SComponent& s, const RComponent& r, const Symbol& input, PDA& rdpda)
{
    string name = "(" + buildStateName(s, r) + ")";
    name += ", " + input.getSymbol();
    try {
        rdpda.addStackSymbol(name);
    } catch (AutomatonException e) {
        /* Stack symbol already exists */
    }
    return name;
}


vector<SComponent> RhdpdaUtils::generateAllPossibleSComponents(set<StateSymbolPair>& pairs)
{
     // Generate all possible pairs of pairs
    vector<PairOfPairs> possiblePairsOfPairs;
    for (const auto& pair1 : pairs) {
        for (const auto& pair2 : pairs) {
            possiblePairsOfPairs.push_back(PairOfPairs(pair1, pair2));
        }
    }
    
    // Generate all possible s components
    vector<SComponent> possibleS;
    int possibleSSize = pow(2, possiblePairsOfPairs.size());
    for (int i = 0; i < possibleSSize; i++) {
        set<PairOfPairs> pairsOfPairsSet;
        int mask = 1;
        for (const auto& pairOfPairs : possiblePairsOfPairs) {
            if ((i & mask) != 0) {
                pairsOfPairsSet.insert(pairOfPairs);
            }
            mask <<= 1;
        }
        possibleS.push_back(SComponent(pairsOfPairsSet));
    }

    return possibleS;
}


vector<RComponent> RhdpdaUtils::generateAllPossibleRComponents(set<StateSymbolPair>& pairs)
{
    // Generate all possible r components
    vector<RComponent> possibleR;
    int possibleRSize = pow(2, pairs.size());
    for (int i = 0; i < possibleRSize; i++) {
        set<StateSymbolPair> pairsSet;
        int mask = 1;
        for (const auto& pair : pairs) {
            if ((i & mask) != 0) {
                pairsSet.insert(pair);
            }
            mask <<= 1;
        }
        possibleR.push_back(RComponent(pairsSet));
    }

    return possibleR;
}


StateData* RhdpdaUtils::getUnmarkedState(map<string, StateData>& states)
{
    StateData* unmarkedStateData = NULL;
    for (auto& stateIter : states) {
        if (!stateIter.second.isMarked) {
            unmarkedStateData = &stateIter.second;
            unmarkedStateData->isMarked = true;
            return unmarkedStateData;
        }
    }
    return NULL;
}


set<StateSymbolPair> RhdpdaUtils::combineStatesWithSymbol(const set<State>& states, const Symbol& symbol)
{
    set<StateSymbolPair> pairs;
    for (const auto& state : states) {
        pairs.insert(StateSymbolPair(state, symbol));
    }
    return pairs;
}


set<StateSymbolPair> RhdpdaUtils::buildAllStateSymbolPairs(PDA& rhdpda)
{
    set<StateSymbolPair> allStateSymbolPairs;
    for (const auto& state : rhdpda.getStates()) {
        for (const auto& symbol : rhdpda.getStackAlphabet()) {
            allStateSymbolPairs.insert(StateSymbolPair(state, symbol));
        }
    }
    return allStateSymbolPairs;
}


}
}
