/*
 * FSM.cpp
 *
 *  Created on: Mar 25, 2013
 *      Author: martin
 */

#include "FSM.h"
#include "../AutomatonPrinter.h"
#include "../exception/AutomatonException.h"
#include <ostream>

namespace automaton {

using namespace std;
using namespace alphabet;

FSM::~FSM() {

}

void FSM::removeState(const State& state) {
	for (set<State>::const_iterator initialState = initialStates.begin(); initialState != initialStates.end();
			initialState++) {
		if (*initialState == state) {
			throw AutomatonException("State \"" + state.getName() + "\" is initial state.");
		}
	}

	for (set<State>::const_iterator finalState = finalStates.begin(); finalState != finalStates.end(); finalState++) {
		if (*finalState == state) {
			throw AutomatonException("State \"" + state.getName() + "\" is final state.");
		}
	}

	for (set<TransitionFSM>::const_iterator t = transitions.begin(); t != transitions.end(); t++) {
		if (state == t->getFrom() || state == t->getTo())
			throw AutomatonException("State \"" + state.getName() + "\" is used in transition.");
	}

	int removed = states.erase(state);
	if (!removed)
		throw AutomatonException("State \"" + state.getName() + "\" doesn't exist.");
}

void FSM::removeInputSymbol(const Symbol& symbol) {
	for (set<TransitionFSM>::const_iterator transition = transitions.begin(); transition != transitions.end();
			transition++) {
		if (symbol == transition->getInput())
			throw AutomatonException("Input symbol \"" + symbol.getSymbol() + "\" is used.");
	}

	int removed = inputAlphabet.erase(symbol);
	if (!removed)
		throw AutomatonException("Input symbol \"" + symbol.getSymbol() + "\" doesn't exist.");

}

void FSM::addTransition(const State& from, const Symbol& input, const State& to) {
	addTransition(TransitionFSM(from, input, to));
}

void FSM::addTransition(const TransitionFSM& transition) {
	const State& from = transition.getFrom();
	const Symbol& input = transition.getInput();
	const State& to = transition.getTo();

	if (states.find(from) == states.end())
		throw AutomatonException("State \"" + from.getName() + "\" doesn't exist.");

	if (input.getSymbol() != "") {
		if (inputAlphabet.find(input) == inputAlphabet.end())
			throw AutomatonException("Input symbol \"" + input.getSymbol() + "\" doesn't exist.");
	}

	if (states.find(to) == states.end())
		throw AutomatonException("State \"" + to.getName() + "\" doesn't exist.");

	pair<set<TransitionFSM>::iterator, bool> ret = transitions.insert(transition);
	if (!ret.second)
		throw AutomatonException(
				"Transition (\"" + from.getName() + "\", \"" + input.getSymbol() + "\") -> \"" + to.getName()
						+ "\" already exists.");
}

void FSM::removeTransition(const TransitionFSM& transition) {
	int removed = transitions.erase(transition);
	if (!removed)
		throw AutomatonException(
				"Transition (\"" + transition.getFrom().getName() + "\", \"" + transition.getInput().getSymbol()
						+ "\") -> \"" + transition.getTo().getName() + "\" doesn't exist.");

}

const set<TransitionFSM>& FSM::getTransitions() const {
	return transitions;
}

const set<TransitionFSM> FSM::getTransitionsFromState(const State& from) const {
	if( states.find(from) == states.end())
		throw AutomatonException("State \"" + from.getName() + "\" doesn't exist");

	set<TransitionFSM> transitionsFromState;
	for(auto const& transition : transitions){
		if (transition.getFrom() == from){
			transitionsFromState.insert(transition);
		}
	}

	return transitionsFromState;
}

const set<TransitionFSM> FSM::getTransitionsToState(const State& to) const {
	if( states.find(to) == states.end())
		throw AutomatonException("State \"" + to.getName() + "\" doesn't exist");

	set<TransitionFSM> transitionsToState;
	for(auto const& transition : transitions){
		if (transition.getTo() == to){
			transitionsToState.insert(transition);
		}
	}

	return transitionsToState;
}

void FSM::toXML(ostream& out) const {
	AutomatonPrinter::toXML(*this, out);
}

bool FSM::isEpsilonFree() const {
	for (auto const& transition : transitions) {
		if (transition.getInput() == Symbol("")) {
			return false;
		}
	}
	return true;
}

bool FSM::isDeterministic() const {
	if (initialStates.size() != 1) {
		return false;
	}

	if (!isEpsilonFree()) {
		return false;
	}

	for (auto transition = transitions.begin(); transition != transitions.end(); transition++) {
		auto subTransition = transition;
		subTransition++;
		for (; subTransition != transitions.end(); subTransition++) {
			if (subTransition->getFrom() == transition->getFrom()
					&& subTransition->getInput() == transition->getInput()) {
				return false;
			}
		}
	}

	return true;
}

bool FSM::isTotal() const {
	return isDeterministic() && transitions.size() == inputAlphabet.size() * states.size();
}

} /* namespace automaton */
