/*
 * PDA.h
 *
 *  Created on: Apr 10, 2013
 *      Author: martin
 */

#ifndef PDA_H_
#define PDA_H_

#include <set>
#include <list>
#include "../Automaton.h"
#include "../State.h"
#include "../../alphabet/Symbol.h"
#include "TransitionPDA.h"

namespace automaton {

using namespace std;
using namespace alphabet;

/**
 * Push Down Automaton
 */
class PDA: public Automaton {
protected:
	set<Symbol> stackAlphabet;
	set<TransitionPDA> transitions;
	list<Symbol> startSymbols;
public:
	PDA();

	/**
	 * @copydoc Automaton::removeState(const State&)
	 */
	virtual void removeState(const State& state);

	/**
	 * @copydoc Automaton::removeInputSymbol(const Symbol&)
	 */
	virtual void removeInputSymbol(const Symbol& symbol);

	/**
	 * Adds symbol to the stack alphabet.
	 * @param symbol Symbol to add
	 * @throws AutomatonException when symbol is already present in stack alphabet
	 */
	void addStackSymbol(const Symbol& symbol);

	/**
	 * Removes symbol from the stack alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is used in transition
	 * (popped from or pushed to the stack), when it is a start symbol
	 * or when it is not present in the stack alphabet
	 */
	void removeStackSymbol(const Symbol& symbol);

	/**
	 * @return the stack alphabet
	 */
	const set<Symbol>& getStackAlphabet() const;

	/**
	 * Adds transition to the PDA.
	 * @param transition transition to add
	 * @throws AutomatonException when some part of the transition is not present
	 * in the PDA (state, input symbol, stack symbol) or when transition already exists
	 */
	void addTransition(const TransitionPDA& transition);

	/**
	 * Removes the transition from the PDA.
	 * @param transition transition to remove
	 * @throws AutomatonException when transition is not present in the PDA
	 */
	void removeTransition(const TransitionPDA& transition);

	/**
	 * @return PDA transitions
	 */
	const set<TransitionPDA>& getTransitions() const;

	/**
	 * Sets the list of start symbols. Start symbols are symbols that are pushed
	 * to the stack when PDA is created.
	 * @param start symbols that are pushed to the stack
	 * @throws AutomatonException when any of the symbols is not present in the stack alphabet
	 */
	void setStartSymbols(const list<Symbol>& start);

	/**
	 * @return list of start symbols
	 */
	const list<Symbol>& getStartSymbols() const;

	/**
	 * @copydoc Automaton::toXML(ostream&) const
	 */
	void toXML(ostream& out) const;
};

} /* namespace automaton */
#endif /* PDA_H_ */
