/*
 * UnknownAutomaton.h
 *
 *  Created on: Oct 12, 2013
 *      Author: Martin Zak
 */

#ifndef UNKNOWNAUTOMATON_H_
#define UNKNOWNAUTOMATON_H_

#include <set>
#include <list>

#include "Automaton.h"
#include "State.h"
#include "../alphabet/Symbol.h"
#include "UnknownTransition.h"

namespace automaton {

using namespace std;
using namespace alphabet;

/**
 * Class representing unknown automaton parsed from XML.
 */
class UnknownAutomaton: public Automaton {
protected:
	set<Symbol> stackAlphabet;
	list<Symbol> startSymbols;
	set<Symbol> tapeAlphabet;
	Symbol blankSymbol;

	set<UnknownTransition> transitions;
public:
	UnknownAutomaton();
	virtual ~UnknownAutomaton();

	/**
	 * Removes the state from the automaton.
	 * @param state State to remove
	 * @throws AutomatonException when state is not present in the automaton
	 */
	void removeState(const State& state);

	/**
	 * Removes input symbol from the input alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is not present in input alphabet
	 */
	void removeInputSymbol(const Symbol& symbol);

	/**
	 * @return the stack alphabet
	 */
	const set<Symbol>& getStackAlphabet() const;

	/**
	 * Adds symbol to the stack alphabet.
	 * @param symbol Symbol to add
	 * @throw AutomatonException when Symbol is already present in stack alphabet
	 */
	void addStackSymbol(const Symbol& symbol);

	/**
	 * Removes symbol from the stack alphabet.
	 * @param symbol Symbol to remove
	 * @throw AutomatonException when Symbol is not present in input alphabet
	 */
	void removeStackSymbol(const Symbol& symbol);

	/**
	 * @return list of start symbols
	 */
	const list<Symbol>& getStartSymbols() const;

	/**
	 * Set the start symbol list.
	 * @param newSymbols symbols to set
	 */
	void setStartSymbols(const list<Symbol>& newSymbols);

	/**
	 * @return the tape alphabet
	 */
	const set<Symbol>& getTapeAlphabet() const;

	/**
	 * Adds symbol to the tape alphabet.
	 * @param symbol Symbol to add
	 * @throw AutomatonException when Symbol is already present in tape alphabet
	 */
	void addTapeSymbol(const Symbol& symbol);

	/**
	 * Removes symbol from the tape alphabet.
	 * @param symbol Symbol to remove
	 * @throw AutomatonException when Symbol is not present in tape alphabet
	 */
	void removeTapeSymbol(const Symbol& symbol);

	/**
	 * @return the blank symbol
	 */
	const Symbol& getBlankSymbol() const;

	/**
	 * Sets the blank symbol.
	 * @param symbol the Symbol to set
	 */
	void setBlankSymbol(const Symbol& symbol);

	/**
	 * @return transitions of the automaton
	 */
	const set<UnknownTransition>& getTransitions() const;

	/**
	 * Adds new transition to the automaton.
	 * @param transition UnknownTransition to add
	 * @throws AutomatonException when transition is already present in the automaton
	 */
	void addTransition(const UnknownTransition& transition);

	/**
	 * Removes the transition from the automaton.
	 * @param transition UnknownTransition to remove
	 * @throws AutomatonException when transition is not present in the automaton
	 */
	void removeTransition(const UnknownTransition& transition);

	/**
	 * @copydoc Automaton::toXML(ostream&) const
	 */
	void toXML(std::ostream& out) const;
};

} /* namespace automaton */
#endif /* UNKNOWNAUTOMATON_H_ */
