/*
 * Concatenation.cpp
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#include "Concatenation.h"

namespace regexp {

Concatenation::Concatenation() {

}

Concatenation::Concatenation(const Concatenation& other) {
	for (auto element : other.elements) {
		elements.push_back(std::shared_ptr<RegExpElement>(element->clone()));
	}
}

Concatenation::Concatenation(Concatenation&& other) noexcept : elements(std::move(other.elements)) {
	other.elements.clear();
}

Concatenation& Concatenation::operator=(const Concatenation& other) {
	if(this == &other) {
		return *this;
	}

	*this = Concatenation(other);
	
	return *this;
}

Concatenation& Concatenation::operator=(Concatenation&& other) noexcept {
	std::swap(this->elements, other.elements);
  
	return *this;
}

Concatenation::~Concatenation() noexcept {
	elements.clear();
}

const std::vector<std::shared_ptr<const RegExpElement> > & Concatenation::getElements() const {
	return * reinterpret_cast<const std::vector<std::shared_ptr<const RegExpElement> > * > (&elements);
}

void Concatenation::appendElement(const RegExpElement& element) {
	this->elements.push_back(std::shared_ptr<RegExpElement>(element.clone()));
}

void Concatenation::appendElement(RegExpElement&& element) {
	this->elements.push_back(std::shared_ptr<RegExpElement>(std::move(element).plunder()));
}

RegExpElement* Concatenation::clone() const {
	return new Concatenation(*this);
}

RegExpElement* Concatenation::plunder() && {
	return new Concatenation(std::move(*this));
}

bool Concatenation::operator<(const RegExpElement& other) const {
	return other > *this;
}

bool Concatenation::operator==(const RegExpElement& other) const {
	return other == *this;
}

bool Concatenation::operator>(const RegExpElement& other) const {
	return other < *this;
}


bool Concatenation::operator<(const Concatenation& other) const {
	int thisSize = this->elements.size();
	int otherSize = other.elements.size();
	if(thisSize < otherSize) return true;
	if(thisSize > otherSize) return false;

	auto thisIter = this->elements.begin();
	auto otherIter = other.elements.begin();
	for(; thisIter != this->elements.end(); thisIter++, otherIter++) {
		if(**thisIter != **otherIter) break;
	}
	if(thisIter == this->elements.end()) return false;

	return **thisIter < **otherIter;
}

bool Concatenation::operator==(const Concatenation& other) const {
	if(this->elements.size() != other.elements.size()) return false;

	auto thisIter = this->elements.begin();
	auto otherIter = other.elements.begin();
	for(; thisIter != this->elements.end(); thisIter++, otherIter++) {
		if(**thisIter != **otherIter) return false;
	}

	return true;
}

void Concatenation::operator>>(std::ostream& out) const {
	out << "(Concatenation";
	for(const auto& e : elements)
		out << " " << *e;

	out << ")";
}

void Concatenation::getAlphabet( std::set<alphabet::Symbol> & alphabet ) const {
	for(const auto& child : this->elements)
		child->getAlphabet(alphabet);
}

bool Concatenation::containsEmptyString() const {
	for(const auto& e : elements)
		if( ! e->containsEmptyString())
			return false;

	return true; // concatenation of zero regexps is epsilon
}

bool Concatenation::isEmpty() const {
	for(const auto& e : elements)
		if(e->isEmpty())
			return true;

	return false; // concatenation of zero regexps is epsilon
}

} /* namespace regexp */
