/*
 * RegExp.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_H_
#define REG_EXP_H_

#include <vector>
#include <list>
#include <string>
#include <memory>
#include "RegExpElement.h"
#include "RegExpEmpty.h"
#include "../std/visitor.hpp"

namespace regexp {

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of RegExpElement.
 */
class RegExp : public std::element<RegExp, std::visitor<RegExp> > {
protected:
	std::shared_ptr<RegExpElement> regExp;

public:
	RegExp();
	RegExp(const RegExpElement& regExp);

	/**
	 * Copy constructor.
	 * @param other RegExp to copy
	 */
	RegExp(const RegExp& other);
	RegExp(RegExp&& other) noexcept;
	RegExp& operator =(const RegExp& other);
	RegExp& operator =(RegExp&& other) noexcept;
	~RegExp() noexcept;

	/**
	 * @return Root node of the regular expression tree
	 */
	const std::shared_ptr<const RegExpElement> getRegExp() const;

	/**
	 * Sets the root node of the regular expression tree. Doesn't perform copy of the regExp param,
	 * just stores it!
	 * @param regExp root node to set
	 */
	void setRegExp(const RegExpElement& regExp);
	
	void setRegExp(RegExpElement&& regExp);
	
	/**
	 * Gets alphabet symbols used in RegExp.
	 * @return set of alphabet symbols used in regexp.
	 */
	std::set<alphabet::Symbol> getAlphabet() const;
	
	/**
	 * @return true if regexp represents empty language
	 */
	bool isEmpty() const;

	/**
	 * @return true if regexp matches empty string (epsilon)
	 */
	bool containsEmptyString() const;

	/**
	 * Prints XML representation of the RegExp to the output stream.
	 * @param out output stream to which print the RegExp
	 * @param regexp RegExp to print
	 */
	friend std::ostream& operator<<(std::ostream& out, const RegExp& regexp);

	bool operator<(const RegExp&) const;
	bool operator<=(const RegExp&) const;
	bool operator==(const RegExp&) const;
	bool operator!=(const RegExp&) const;
	bool operator>(const RegExp&) const;
	bool operator>=(const RegExp&) const;
};

} /* namespace regexp */

#endif /* REG_EXP_H_ */
