/*
 * RegExpFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "RegExpFromXMLParser.h"
#include "../sax/ParserException.h"

namespace regexp {

RegExp RegExpFromXMLParser::parse(std::list<sax::Token>& input) {
	popToken(input, sax::Token::START_ELEMENT, "regexp");
	RegExp regexp;
	RegExpElement* element = parseElement(input);
	if(element == NULL) throw sax::ParserException(sax::Token("", sax::Token::CHARACTER), input.front());
	regexp.setRegExp(std::move(*element));
	delete element;
	popToken(input, sax::Token::END_ELEMENT, "regexp");

	return regexp;
}

RegExpElement* RegExpFromXMLParser::parseElement(std::list<sax::Token>& input) {
	if (isToken(input, sax::Token::START_ELEMENT, "symbol")) {
		return parseSymbol(input);
	} else if (isToken(input, sax::Token::START_ELEMENT, "empty")) {
		return parseEmpty(input);
	} else if (isToken(input, sax::Token::START_ELEMENT, "epsilon")) {
		return parseEpsilon(input);
	} else if (isToken(input, sax::Token::START_ELEMENT, "iteration")) {
		return parseIteration(input);
	} else if (isToken(input, sax::Token::START_ELEMENT, "alternation")) {
		return parseAlternation(input);
	} else if (isToken(input, sax::Token::START_ELEMENT, "concatenation")) {
		return parseConcatenation(input);
	} else {
		return NULL;
	}
}

Alternation* RegExpFromXMLParser::parseAlternation(std::list<sax::Token>& input) {
	popToken(input, sax::Token::START_ELEMENT, "alternation");

	Alternation* alternation = new Alternation;
	while (true) {
		RegExpElement* element = parseElement(input);
		if(!element) break;
		alternation->insertElement(std::move(*element));
		delete element;
	}

	popToken(input, sax::Token::END_ELEMENT, "alternation");
	return alternation;
}

Concatenation* RegExpFromXMLParser::parseConcatenation(std::list<sax::Token>& input) {
	popToken(input, sax::Token::START_ELEMENT, "concatenation");

	Concatenation* concatenation = new Concatenation();
	while (true) {
		RegExpElement* element = parseElement(input);
		if(!element) break;
		concatenation->appendElement(std::move(*element));
		delete element;
	}

	popToken(input, sax::Token::END_ELEMENT, "concatenation");
	return concatenation;

}

Iteration* RegExpFromXMLParser::parseIteration(std::list<sax::Token>& input) {
	popToken(input, sax::Token::START_ELEMENT, "iteration");

	Iteration* iteration = new Iteration();
	RegExpElement* element = parseElement(input);
	if(element == NULL) throw sax::ParserException(sax::Token("", sax::Token::CHARACTER), input.front());
	iteration->setElement(std::move(*element));
	delete element;

	popToken(input, sax::Token::END_ELEMENT, "iteration");
	return iteration;
}

RegExpEpsilon* RegExpFromXMLParser::parseEpsilon(std::list<sax::Token>& input) {
	popToken(input, sax::Token::START_ELEMENT, "epsilon");

	RegExpEpsilon* epsilon = new RegExpEpsilon();

	popToken(input, sax::Token::END_ELEMENT, "epsilon");
	return epsilon;
}

RegExpEmpty* RegExpFromXMLParser::parseEmpty(std::list<sax::Token>& input) {
	popToken(input, sax::Token::START_ELEMENT, "empty");

	RegExpEmpty* empty = new RegExpEmpty();

	popToken(input, sax::Token::END_ELEMENT, "empty");
	return empty;
}

RegExpSymbol* RegExpFromXMLParser::parseSymbol(std::list<sax::Token>& input) {
	popToken(input, sax::Token::START_ELEMENT, "symbol");

	if (input.front().getType() == sax::Token::CHARACTER) {
		RegExpSymbol* symbol = new RegExpSymbol(input.front().getData());
		input.pop_front();
		popToken(input, sax::Token::END_ELEMENT, "symbol");
		return symbol;
	} else {
		throw sax::ParserException(sax::Token("", sax::Token::CHARACTER), input.front());
	}
}

bool RegExpFromXMLParser::isToken(std::list<sax::Token>& input, sax::Token::TokenType type, std::string data) {
	return input.front().getType() == type && input.front().getData() == data;
}

void RegExpFromXMLParser::popToken(std::list<sax::Token>& input, sax::Token::TokenType type, std::string data) {
	if (isToken(input, type, data)) {
		input.pop_front();
	} else {
		throw sax::ParserException(sax::Token(data, type), input.front());
	}
}

} /* namespace regexp */

